"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Certificate = void 0;
const client_acm_1 = require("@aws-sdk/client-acm");
const client_route_53_1 = require("@aws-sdk/client-route-53");
/**
 * @class 	Certificate
 * @desc 	Represents a certificate
 */
class Certificate {
    constructor(region, id) {
        this.domain = [];
        this.region = region;
        this.id = id;
        // this.domain = domain
        // this.arn = arn
    }
    setDomain(...domain) {
        this.domain = domain;
        return this;
    }
    request() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.send(new client_acm_1.RequestCertificateCommand({
                // First domain in array is
                DomainName: this.domain[0].getName(),
                SubjectAlternativeNames: this.domain.length > 1 ?
                    this.domain.slice(1).map((domain) => domain.getName()) : undefined,
                ValidationMethod: 'DNS'
            }));
            this.id = result.CertificateArn;
            return this.validate();
        });
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.send(new client_acm_1.DescribeCertificateCommand({
                CertificateArn: this.id
            }));
            if (result.Certificate) {
                const certificate = result.Certificate;
                this.detail = certificate;
                this.status = certificate.Status;
            }
            return this;
        });
    }
    validate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id)
                throw new Error('invalid ARN'); // TODO
            yield this.get();
            // Validate each record in the domain validation options
            if (this.detail && this.detail.DomainValidationOptions != null) {
                const records = [];
                const recordName = new Map();
                for (const validation of this.detail.DomainValidationOptions) {
                    const record = validation.ResourceRecord;
                    if (!record || !record.Name || !record.Value)
                        continue;
                    if (!recordName.has(record.Name) || !recordName.get(record.Name).has(record.Value)) {
                        records.push(record);
                        recordName.set(record.Name, new Set([record.Value]));
                    }
                }
                yield Certificate.getDomainClient().send(new client_route_53_1.ChangeResourceRecordSetsCommand({
                    ChangeBatch: {
                        Changes: records.map((record) => ({
                            Action: 'CREATE',
                            ResourceRecordSet: {
                                Name: record.Name,
                                ResourceRecords: [
                                    { Value: record.Value }
                                ],
                                TTL: 60,
                                Type: record.Type
                            }
                        })),
                        Comment: 'ACM validation'
                    },
                    HostedZoneId: this.domain[0].getId()
                }));
            }
        });
    }
    /**
     * Returns the viewer certificate configuration from CloudFront
     */
    getViewerCertificate() {
        return {
            CloudFrontDefaultCertificate: false,
            ACMCertificateArn: this.getArn(),
            Certificate: this.getArn(),
            CertificateSource: 'acm',
            MinimumProtocolVersion: 'TLSv1.1_2016',
            SSLSupportMethod: 'sni-only'
        };
    }
    setDetail(detail) {
        this.detail = detail;
        return this;
    }
    getArn() {
        return this.id;
    }
    setArn(arn) {
        this.id = arn;
        return this;
    }
    isIssued() {
        return this.status === 'ISSUED';
    }
    isInactive() {
        return this.status === 'INACTIVE';
    }
    isExpired() {
        return this.status === 'EXPIRED';
    }
    isRevoked() {
        return this.status === 'REVOKED';
    }
    isFailed() {
        return this.status === 'FAILED';
    }
    validationPending() {
        return this.status === 'PENDING_VALIDATION';
    }
    validationTimedOut() {
        return this.status === 'VALIDATION_TIMED_OUT';
    }
    getClient() {
        return this.region.getClient();
    }
    static getDomainClient() {
        if (!Certificate.domainClient)
            Certificate.domainClient = new client_route_53_1.Route53Client({});
        return Certificate.domainClient;
    }
}
exports.Certificate = Certificate;
