"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudFrontSocket = exports.CloudFrontServer = void 0;
const http_1 = __importDefault(require("http"));
const crypto_1 = __importDefault(require("crypto"));
const proxy_1 = require("./proxy");
class CloudFrontServer {
    constructor(dir, handler, port = 3000) {
        this.alive = false;
        this.dir = dir;
        this.handler = handler;
        this.port = port;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            // The app router should handle all errors and always produce a response,
            // so just adds one error handler to the top-level Promise to ensure
            // a response is always sent to every incoming request
            this.server = http_1.default.createServer((req, res) => {
                (0, proxy_1.httpToProxyEventV2)(req).then((event) => {
                    // send event to lambda handler
                    // proxyResultToHttpV2(res)
                });
            });
            // Listen on the configured port and maintain active socket mapping
            this.server.listen(this.port, () => {
                this.onStart();
            });
            // Initial socket mapping to TCP socket
            this.server.on('connect', (_, socket) => {
                const tcp = new TCPSocket(this, socket);
                this.addSocket(tcp);
            });
            // Upgrade request to WebSocket
            this.server.on('upgrade', (request, socket) => {
                const ws = new WebSocket(this, request, socket);
                ws.start().then(() => {
                    this.addSocket(ws);
                    this.routeWebsocketConnect(ws);
                });
            });
            // Set the alive flag to indicate all initialization went through
            this.alive = true;
            // Listen to shutdown signals and close the socket
            process.on('SIGTERM', () => this.stop());
            process.on('SIGINT', () => this.stop());
        });
    }
    /**
     * @func 	{addSocket}
     * @desc 	Adds a socket to the mapping of managed sockets with the given random ID.
     */
    addSocket(socket) {
        if (!this.socket)
            this.socket = new Map();
        this.socket.set(socket.getId(), socket);
    }
    /**
     * @func 	{removeSocket}
     * @desc 	Remove the given socket from the server mapping.
     */
    removeSocket(socket) {
        if (this.socket)
            this.socket.delete(socket.getId());
    }
    /**
     * @func 	{stop}
     * @desc 	Stops the server and tries to gracefully close all connections within the
     * 			force close timeout window.
     */
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.alive || !this.server)
                return true;
            this.alive = false;
            // Close all open connections
            this.server.closeAllConnections();
            console.log('ending with timeout', SERVER_CLOSE_TIMEOUT);
            return new Promise((resolve) => {
                // Force close the server and destroy all sockets
                const closeTimeout = setTimeout(() => {
                    console.log('force close');
                    this.server.unref();
                    if (this.socket)
                        for (const socket of this.socket.values())
                            socket.destroy();
                    resolve(false);
                    process.exit(0);
                }, SERVER_CLOSE_TIMEOUT);
                // Attempt to close all open connections gracefully
                try {
                    this.server.close((error) => {
                        clearTimeout(closeTimeout);
                        this.server.unref();
                        resolve(error == null);
                    });
                }
                catch (error) {
                    clearTimeout(closeTimeout);
                    try {
                        this.server.unref();
                        resolve(true);
                    }
                    catch (error) {
                        resolve(false);
                    }
                }
            });
        });
    }
    onStart() {
        console.log('Server started on ' + this.port);
        // Check for any context methods with start
    }
    onIncoming(req) {
    }
    onRequest(request) {
    }
    onResult(result) {
    }
    onRequestError(error) {
    }
}
exports.CloudFrontServer = CloudFrontServer;
class CloudFrontSocket {
    constructor(socket) {
        this.stopping = false;
        this.stopped = false;
        this.id = crypto_1.default.randomBytes(16).toString('hex');
        this.router = router;
        this.socket = socket;
    }
    stop(error) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stopped || this.stopping)
                return true;
            this.stopping = true;
            if (error)
                console.log('stop socket error', error);
            // if (this.ended())
            // 	return true
            return new Promise((resolve, reject) => {
                this.socket.end(error ? error.toString() : undefined, () => {
                    console.log('ended socket');
                    this.destroy();
                    if (error)
                        reject(error);
                    else
                        resolve(true);
                });
            });
        });
    }
    destroy() {
        if (!this.stopped) {
            this.stopped = true;
            this.socket.destroy();
        }
        return this;
    }
    getId() {
        return this.id;
    }
    ended() {
        return this.socket.readableEnded;
    }
}
exports.CloudFrontSocket = CloudFrontSocket;
