"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.proxyResultToHttp = exports.httpToProxyEvent = exports.proxyServer = void 0;
const http_1 = __importDefault(require("http"));
const url_1 = __importDefault(require("url"));
const crypto_1 = __importDefault(require("crypto"));
const BODY_METHODS = new Set(['POST', 'PATCH', 'PUT']);
/**
 *
 * https://docs.aws.amazon.com/lambda/latest/dg/urls-invocation.html#urls-payloads
 */
function proxyServer(handler, port = 3000) {
    return __awaiter(this, void 0, void 0, function* () {
        const delimiter = handler.lastIndexOf('.');
        const handlerPath = handler.substring(0, delimiter);
        const handlerFunction = handler.substring(delimiter + 1);
        const server = http_1.default.createServer((req, res) => {
            httpToProxyEvent(req)
                .then((event) => {
                delete require.cache[require.resolve(handlerPath)];
                return require(handlerPath)[handlerFunction](event);
            })
                .then((result) => {
                return proxyResultToHttp(result, res);
            })
                .then((proxied) => {
                // log request
            })
                .catch((error) => {
                console.log('error', error);
                // res.writeHead(500).end(error.toString())
            });
        });
        process.on('SIGTERM', () => stopServer());
        process.on('SIGINT', () => stopServer());
        const stopServer = function () {
            server.closeAllConnections();
        };
        return new Promise((resolve) => {
            server.listen(port, () => {
                console.log('http://localhost:' + port);
                resolve(server);
            });
        });
    });
}
exports.proxyServer = proxyServer;
function httpToProxyEvent(req) {
    return __awaiter(this, void 0, void 0, function* () {
        const method = httpRequestMethod(req);
        const { rawPath, rawQueryString, queryStringParameters } = httpRequestQuery(req);
        const headers = httpRequestHeaders(req);
        const body = yield httpRequestBody(req);
        // Construct proxy object
        return {
            version: '2.0',
            routeKey: '$default',
            rawPath,
            rawQueryString,
            cookies: [],
            headers,
            queryStringParameters,
            body,
            isBase64Encoded: false,
            stageVariables: {},
            requestContext: {
                accountId: process.env.AWS_ACCOUNT || 'AWS_ACCOUNT',
                apiId: '<url ID>',
                routeKey: '$default',
                requestId: crypto_1.default.randomUUID(),
                stage: '$default',
                time: '',
                timeEpoch: 12,
                http: {
                    method,
                    path: rawPath,
                    protocol: 'HTTP/1.1',
                    sourceIp: req.socket.remoteAddress || '',
                    userAgent: req.headers['user-agent'] || ''
                },
                domainName: '',
                domainPrefix: ''
            }
        };
    });
}
exports.httpToProxyEvent = httpToProxyEvent;
/**
 * Returns the uppercase HTTP method (GET, POST, ...)
 */
function httpRequestMethod(req) {
    // Parse request method
    if (!req.method)
        throw new Error('invalid request method');
    return req.method.toUpperCase();
}
function httpRequestQuery(req) {
    if (!req.url)
        throw new Error('invalid URL');
    // Get path from URL
    const target = url_1.default.parse(req.url);
    if (!target.pathname)
        throw new Error('invalid path');
    const rawPath = target.pathname;
    // Parse query string parameters
    const rawQueryString = target.query || '';
    const queryStringParameters = {};
    if (target.query)
        for (const [key, value] of new URLSearchParams(target.query))
            queryStringParameters[key] = Array.isArray(value) ? value[0] : value;
    return {
        rawPath,
        rawQueryString,
        queryStringParameters
    };
}
function httpRequestHeaders(req) {
    const headers = {};
    Object.keys(req.headers).forEach((key) => {
        const value = req.headers[key];
        if (value)
            headers[key] = Array.isArray(value) ? value[0] : value;
    });
    return headers;
}
function httpRequestBody(req) {
    return __awaiter(this, void 0, void 0, function* () {
        // Parse body string
        let body = undefined;
        if (req.method && BODY_METHODS.has(req.method.toUpperCase())) {
            body = yield new Promise((resolve) => {
                const chunks = [];
                req.on('data', (chunk) => {
                    chunks.push(chunk);
                });
                req.on('end', () => {
                    resolve(chunks.join(''));
                });
            });
        }
        return body;
    });
}
function proxyResultToHttp(result, res) {
    return __awaiter(this, void 0, void 0, function* () {
        res.setHeader('Connection', 'close');
        if (typeof result === 'string') {
            res.writeHead(200);
            res.write(result);
        }
        else {
            res.writeHead(result.statusCode || 200);
            res.write(result.body);
            if (result.headers) {
                for (const key of Object.keys(result.headers)) {
                    const value = result.headers[key];
                    res.setHeader(key, value.toString());
                }
            }
        }
        res.end();
    });
}
exports.proxyResultToHttp = proxyResultToHttp;
